<?php

/**
 * Build a batch to rebuild all the translation information.
 */
function active_translation_build_batch_all() {
  $batch = array(
    'title' => t('Rebuilding the active translation table'),
    'operations' => array(
      array('_active_translation_batch_operation', array()),
    ),
    'finished' => '_active_translation_batch_finished',
    'file' => drupal_get_path('module', 'active_translation') . '/active_translation.batch.inc',
  );

  return $batch;
}

function _active_translation_batch_operation(&$context) {
  if (empty($context['sandbox'])) {
    // Initiate multistep processing.
    $context['sandbox']['languages'] = array_keys(locale_language_list('name', TRUE));
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['current_translation'] = 0;
    $context['sandbox']['max'] = db_result(db_query('SELECT COUNT(DISTINCT tnid) FROM {node} WHERE tnid != 0'));

    // Remove all existing translation information.
    db_query('DELETE FROM {active_translation}');

    // Insert records for language neutral and untranslated nodes.
    $fields = $values = array();
    // We need to put the node id into the column for each language.
    foreach ($context['sandbox']['languages'] as $lang_code) {
      $fields[] = db_escape_table($lang_code);
      $values[] = 'nid';
    }
    db_query('INSERT INTO {active_translation} (atid, '. implode(', ', $fields) .') SELECT DISTINCT(nid), '. implode(', ', $values) .' FROM {node} WHERE tnid = 0 ORDER BY tnid');
  }

  // Process the next set of translations.
  $limit = 10;
  $result = db_query_range("SELECT DISTINCT tnid FROM {node} WHERE tnid > %d AND tnid != 0 ORDER BY tnid ASC", $context['sandbox']['current_translation'], 0, $limit);
  while ($row = db_fetch_object($result)) {
    $node = node_load($row->tnid);
    $translation = active_translation_recompute($node);
    drupal_write_record('active_translation', $translation);

    $context['sandbox']['progress']++;
    $context['sandbox']['current_translation'] = $row->tnid;
  }

  // Multistep processing : report progress.
  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

function _active_translation_batch_finished($success, $results, $operations) {
  if ($success) {
    drupal_set_message(t('The active translation table has rebuilt.'));
    node_access_needs_rebuild(FALSE);
  }
  else {
    drupal_set_message(t('The active translation table has not been properly rebuilt.'), 'error');
  }
  cache_clear_all();
}

